unit TokenizerClass;

interface

uses
  Classes, TokenizationClass;

type
  TTokenizer = class
  private
    FErrorLine: Integer;
  public
    property ErrorLine: Integer read FErrorLine;
    function Process(S: TStream; T: TTokenization; HandleToken: TTokenHandler): Boolean;
    function ProcessString(const S: String; T: TTokenization; HandleToken: TTokenHandler): Boolean;
    function ProcessFile(const FileName: String; T: TTokenization; HandleToken: TTokenHandler): Boolean;
  end;

implementation

uses
  SysUtils;
  
function TTokenizer.Process(S: TStream; T: TTokenization; HandleToken: TTokenHandler): Boolean;
var
  C: Char;
  R: Integer;
begin
  Result := True;
  if Assigned(HandleToken) then
  begin
    R := 0;
    T.Initialize;
    S.Seek(0, soBeginning);
    while S.Position < S.Size do
    begin
      S.Read(C, SizeOf(Char));
      if not T.ProcessCharacter(C, HandleToken, R) then
      begin
        FErrorLine := T.CurrentLine;
        Result := False;
        Exit;
      end
      else if R <> 0 then
      begin
        S.Seek(-R, soFromCurrent);
        R := 0;
      end;
    end;
    T.ProcessCharacter(#0, HandleToken, R);
  end;
end;

function TTokenizer.ProcessString(const S: String; T: TTokenization; HandleToken: TTokenHandler): Boolean;
var
  Stream: TStream;
begin
  Stream := TStringStream.Create(S);
  try
    Result := Process(Stream, T, HandleToken);
  finally
    Stream.Free;
  end;
end;

function TTokenizer.ProcessFile(const FileName: String; T: TTokenization; HandleToken: TTokenHandler): Boolean;
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    Result := Process(Stream, T, HandleToken);
  finally
    Stream.Free;
  end;
end;

end.
