program StringEvaluatorTest;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Classes,
  StringEvaluatorClass,
  TokenizerClass in '..\..\Include\TokenizerClass.pas',
  TokenizationClass in '..\..\Include\TokenizationClass.pas';

procedure Verify(const Expression: String; Strings: Array of String; Result: Boolean);
const
  B2S: Array[Boolean] of String = ('F', 'T');
var
  S: String;
  i: Integer;
  SL: TStrings;
  SE: TStringEvaluator;
begin
  SL := TStringList.Create;
  SE := TStringEvaluator.Create;
  try
    SE.CreateBooleanTree(Expression);
    for i := 0 to Length(Strings)-1 do
      SL.Add(Strings[i]);
    if SE.Evaluate(SL) = Result then
      S := '---   OK(%s)   --- %s. Strings: %s'
    else
      S := '--- ERROR(%s)! --- %s. Strings: %s';
    WriteLn(Format(S, [B2S[Result], Expression, SL.CommaText]));
  finally
    SE.Free;
    SL.Free;
  end;
end;

begin
  Verify('Sweden', ['Sweden'], True);
  Verify('Sweden', ['Denmark'], False);
  Verify('Sweden or Denmark', ['Sweden'], True);
  Verify('Sweden or Denmark or Germany', ['Sweden'], True);
  Verify('Sweden or Denmark', ['Denmark'], True);
  Verify('Sweden and Denmark', ['Sweden'], False);
  Verify('Sweden and Denmark', ['Denmark'], False);
  Verify('Sweden and Denmark', ['Sweden', 'Denmark'], True);
  Verify('Sweden and not Denmark', ['Sweden'], True);
  Verify('Sweden and not Denmark', ['Denmark'], False);
  Verify('Sweden and not Denmark', ['Sweden', 'Denmark'], False);
  Verify('Sweden or Denmark and not Germany', ['Sweden'], True);
  Verify('Sweden or Denmark and not Germany', ['Denmark'], True);
  Verify('Sweden or Denmark and not Germany', ['Denmark', 'Germany'], False);
  Verify('Sweden or (Denmark and not Germany)', ['Sweden'], True);
  Verify('Sweden or (Denmark and not Germany)', ['Denmark'], True);
  Verify('Sweden or (Denmark and not Germany)', ['Denmark', 'Germany'], False);
  Verify('(Sweden or Denmark) and not Germany', ['Sweden'], True);
  Verify('(Sweden or Denmark) and not Germany', ['Denmark'], True);
  Verify('(Sweden or Denmark) and not Germany', ['Denmark', 'Germany'], False);
  Verify('and or or', ['and'], True);
  Verify('and or or', ['or'], True);
  Verify('and or or', ['and', 'or'], True);
  Verify('and and or', ['and'], False);
  Verify('and and or', ['or'], False);
  Verify('and and or', ['and', 'or'], True);
  ReadLn;
end.
