unit StringEvaluatorTokenizationClass;

interface

uses
  TokenizationClass;

type
  TsetState = (setParsingNothing, setParsingString, setParsingQuotedString);

  TStringEvaluatorTokenization = class(TTokenization)
  protected
    State: TsetState;
  public
    procedure Initialize; override;
    function ProcessCharacter(C: Char; HandleToken: TTokenHandler; var Reverse: Integer): Boolean; override;
  end;

implementation

{$ifdef VER150} {$define OLDVER} {$endif}
{$ifdef VER160} {$define OLDVER} {$endif}
{$ifdef VER170} {$define OLDVER} {$endif}
{$ifdef VER180} {$define OLDVER} {$endif}
{$ifdef VER185} {$define OLDVER} {$endif}
{$ifdef OLDVER}
type TCharSet = Set of Char;
function CharInSet(C: AnsiChar; CharSet: TCharSet): Boolean;
begin
  Result := C in CharSet;
end;
{$endif}

procedure TStringEvaluatorTokenization.Initialize;
begin
  inherited;
  State := setParsingNothing;
end;

function TStringEvaluatorTokenization.ProcessCharacter(C: Char; HandleToken: TTokenHandler;
  var Reverse: Integer): Boolean;
type
  TCharType = (ctLPar, ctRPar, ctSpace, ctQuote, ctAlpha, ctLineBreak, ctOther);

  function CharType: TCharType;
  begin
    if C = '(' then
      Result := ctLPar
    else if C = ')' then
      Result := ctRPar
    else if CharInSet(C, [' ', #0, #13]) then
      Result := ctSpace
    else if C = #10 then
      Result := ctLineBreak
    else if C = '"' then
      Result := ctQuote
    else if CharInSet(C, ['a'..'z', 'A'..'Z']) then
      Result := ctAlpha
    else
      Result := ctOther;
  end;

begin
  Result := True;
  case State of
    setParsingNothing:
      case CharType of
        ctLPar, ctRPar:
        begin
          AddToToken(C);
          DoToken(HandleToken, C);
        end;
        ctSpace:
          ;
        ctQuote:
          State := setParsingQuotedString;
        ctAlpha:
        begin
          AddToToken(C);
          State := setParsingString;
        end;
        ctLineBreak:
          IncreaseLine;
      else
        Result := False;
      end;
    setParsingQuotedString:
      case CharType of
        ctQuote:
        begin
          DoToken(HandleToken, C);
          State := setParsingNothing;
        end;
        ctSpace, ctAlpha:
          AddToToken(C);
        ctLineBreak:
          IncreaseLine;
      else
        Result := False;
      end;
    setParsingString:
      case CharType of
        ctLPar, ctRPar:
        begin
          DoToken(HandleToken, C);
          AddToToken(C);
          DoToken(HandleToken, C);
          State := setParsingNothing;
        end;
        ctAlpha:
          AddToToken(C);
        ctSpace:
        begin
          DoToken(HandleToken, C);
          State := setParsingNothing;
        end;
        ctLineBreak:
          IncreaseLine;
      else
        Result := False;
      end;
  end;
end;

end.
