unit TokenizerFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    PerformCodeTokenizationButton: TButton;
    CodeSourceMemo: TMemo;
    TokensMemo: TMemo;
    TokenizationRadioGroup: TRadioGroup;
    TextSourceMemo: TMemo;
    PerformTextTokenizationButton: TButton;
    PerformFileTokenizationButton: TButton;
    OpenDialog: TOpenDialog;
    procedure PerformCodeTokenizationButtonClick(Sender: TObject);
    procedure PerformTextTokenizationButtonClick(Sender: TObject);
    procedure PerformFileTokenizationButtonClick(Sender: TObject);
  protected
    procedure HandleToken(Sender: TObject; const Token: String; Line: Integer);
    procedure PerformTokenization(const S: String; IsFileName: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  TokenizerClass, TokenizationClass, LineTokenizationClass, WordTokenizationClass,
  PartialDelphiTokenizationClass, StringEvaluatorTokenizationClass;

type
  TTokenizationClass = class of TTokenization;

procedure TForm1.HandleToken(Sender: TObject; const Token: String; Line: Integer);
begin
  TokensMemo.Lines.Add(Format('%d: %s', [Line, Token]));
end;

procedure TForm1.PerformTokenization(const S: String; IsFileName: Boolean);
var
  Tokenizer: TTokenizer;
  Tokenization: TTokenization;
  TokenizationClass: TTokenizationClass;

  function Process: Boolean;
  begin
    if IsFileName then
      Result := Tokenizer.ProcessFile(S, Tokenization, HandleToken)
    else
      Result := Tokenizer.ProcessString(S, Tokenization, HandleToken)
  end;

begin
  Tokenizer := nil;
  Tokenization := nil;
  case TokenizationRadioGroup.ItemIndex of
    0: TokenizationClass := TLineTokenization;
    1: TokenizationClass := TWordTokenization;
    2: TokenizationClass := TPartialDelphiTokenization;
  else
    TokenizationClass := TStringEvaluatorTokenization;
  end;
  try
    TokensMemo.Clear;
    Tokenizer := TTokenizer.Create;
    Tokenization := TokenizationClass.Create;
    if not Process then
      ShowMessage(Format('Error in line %d', [Tokenizer.ErrorLine]));
  finally
    Tokenization.Free;
    Tokenizer.Free;
  end;
end;

procedure TForm1.PerformCodeTokenizationButtonClick(Sender: TObject);
begin
  PerformTokenization(CodeSourceMemo.Lines.Text, False);
end;

procedure TForm1.PerformTextTokenizationButtonClick(Sender: TObject);
begin
  PerformTokenization(TextSourceMemo.Lines.Text, False);
end;

procedure TForm1.PerformFileTokenizationButtonClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    PerformTokenization(OpenDialog.FileName, True);
end;


end.
